# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-05-29 10:26
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_p', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='barrier',
            name='target',
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE, to='django_p.Pipeline'),
        ),
    ]
