# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-06-02 09:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_p', '0004_auto_20160529_1059'),
    ]

    operations = [
        migrations.AddField(
            model_name='pipeline',
            name='output',
            field=models.ForeignKey(
                null=True, on_delete=django.db.models.deletion.CASCADE, to='django_p.Slot'),
        ),
        migrations.AlterUniqueTogether(
            name='slot',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='slot',
            name='name',
        ),
    ]
