# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-05-29 10:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_p', '0003_auto_20160529_1027'),
    ]

    operations = [
        migrations.AlterField(
            model_name='barrier',
            name='triggered',
            field=models.DateTimeField(null=True),
        ),
    ]
